import type { AxiosError, AxiosInstance, AxiosResponse, AxiosStatic, CreateAxiosDefaults } from 'axios';
import type { BodySerializer, QuerySerializer, QuerySerializerOptions } from './utils';
type OmitKeys<T, K> = Pick<T, Exclude<keyof T, K>>;
export interface Config<ThrowOnError extends boolean = boolean> extends Omit<CreateAxiosDefaults, 'auth' | 'headers'> {
    auth?: ((auth: Auth) => Promise<AuthToken> | AuthToken) | AuthToken;
    axios?: AxiosStatic;
    bodySerializer?: BodySerializer;
    headers?: CreateAxiosDefaults['headers'] | Record<string, string | number | boolean | (string | number | boolean)[] | null | undefined | unknown>;
    method?: 'connect' | 'delete' | 'get' | 'head' | 'options' | 'patch' | 'post' | 'put' | 'trace';
    querySerializer?: QuerySerializer | QuerySerializerOptions;
    responseTransformer?: (data: unknown) => Promise<unknown>;
    responseValidator?: (data: unknown) => Promise<unknown>;
    throwOnError?: ThrowOnError;
}
export interface Auth {
    in?: 'header' | 'query';
    name?: string;
    scheme?: 'basic' | 'bearer';
    type: 'apiKey' | 'http';
}
type AuthToken = string | undefined;
export interface RequestOptions<ThrowOnError extends boolean = boolean, Url extends string = string> extends Config<ThrowOnError> {
    body?: unknown;
    client?: Client;
    path?: Record<string, unknown>;
    query?: Record<string, unknown>;
    security?: ReadonlyArray<Auth>;
    url: Url;
}
export type RequestResult<Data = unknown, TError = unknown, ThrowOnError extends boolean = boolean> = ThrowOnError extends true ? Promise<AxiosResponse<Data>> : Promise<(AxiosResponse<Data> & {
    error: undefined;
}) | (AxiosError<TError> & {
    data: undefined;
    error: TError;
})>;
type MethodFn = <Data = unknown, TError = unknown, ThrowOnError extends boolean = false>(options: Omit<RequestOptions<ThrowOnError>, 'method'>) => RequestResult<Data, TError, ThrowOnError>;
type RequestFn = <Data = unknown, TError = unknown, ThrowOnError extends boolean = false>(options: Omit<RequestOptions<ThrowOnError>, 'method'> & Pick<Required<RequestOptions<ThrowOnError>>, 'method'>) => RequestResult<Data, TError, ThrowOnError>;
export interface Client {
    buildUrl: <Data extends {
        body?: unknown;
        path?: Record<string, unknown>;
        query?: Record<string, unknown>;
        url: string;
    }>(options: Pick<Data, 'url'> & Omit<Options<Data>, 'axios'>) => string;
    delete: MethodFn;
    get: MethodFn;
    getConfig: () => Config;
    head: MethodFn;
    instance: AxiosInstance;
    options: MethodFn;
    patch: MethodFn;
    post: MethodFn;
    put: MethodFn;
    request: RequestFn;
    setConfig: (config: Config) => Config;
}
interface DataShape {
    body?: unknown;
    headers?: unknown;
    path?: unknown;
    query?: unknown;
    url: string;
}
export type Options<Data extends DataShape = DataShape, ThrowOnError extends boolean = boolean> = OmitKeys<RequestOptions<ThrowOnError>, 'body' | 'path' | 'query' | 'url'> & Omit<Data, 'url'>;
export type OptionsLegacyParser<Data = unknown, ThrowOnError extends boolean = boolean> = Data extends {
    body?: any;
} ? Data extends {
    headers?: any;
} ? OmitKeys<RequestOptions<ThrowOnError>, 'body' | 'headers' | 'url'> & Data : OmitKeys<RequestOptions<ThrowOnError>, 'body' | 'url'> & Data & Pick<RequestOptions<ThrowOnError>, 'headers'> : Data extends {
    headers?: any;
} ? OmitKeys<RequestOptions<ThrowOnError>, 'headers' | 'url'> & Data & Pick<RequestOptions<ThrowOnError>, 'body'> : OmitKeys<RequestOptions<ThrowOnError>, 'url'> & Data;
export {};
