"use strict";
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createConfig = exports.urlSearchParamsBodySerializer = exports.jsonBodySerializer = exports.formDataBodySerializer = exports.mergeHeaders = exports.axiosHeadersKeywords = exports.mergeConfigs = exports.getUrl = exports.buildUrl = exports.setAuthParams = exports.getAuthToken = exports.createQuerySerializer = void 0;
const PATH_PARAM_RE = /\{[^{}]+\}/g;
const serializePrimitiveParam = ({ allowReserved, name, value, }) => {
    if (value === undefined || value === null) {
        return '';
    }
    if (typeof value === 'object') {
        throw new Error('Deeply-nested arrays/objects aren’t supported. Provide your own `querySerializer()` to handle these.');
    }
    return `${name}=${allowReserved ? value : encodeURIComponent(value)}`;
};
const separatorArrayExplode = (style) => {
    switch (style) {
        case 'label':
            return '.';
        case 'matrix':
            return ';';
        case 'simple':
            return ',';
        default:
            return '&';
    }
};
const separatorArrayNoExplode = (style) => {
    switch (style) {
        case 'form':
            return ',';
        case 'pipeDelimited':
            return '|';
        case 'spaceDelimited':
            return '%20';
        default:
            return ',';
    }
};
const separatorObjectExplode = (style) => {
    switch (style) {
        case 'label':
            return '.';
        case 'matrix':
            return ';';
        case 'simple':
            return ',';
        default:
            return '&';
    }
};
const serializeArrayParam = ({ allowReserved, explode, name, style, value, }) => {
    if (!explode) {
        const joinedValues = (allowReserved ? value : value.map((v) => encodeURIComponent(v))).join(separatorArrayNoExplode(style));
        switch (style) {
            case 'label':
                return `.${joinedValues}`;
            case 'matrix':
                return `;${name}=${joinedValues}`;
            case 'simple':
                return joinedValues;
            default:
                return `${name}=${joinedValues}`;
        }
    }
    const separator = separatorArrayExplode(style);
    const joinedValues = value
        .map((v) => {
        if (style === 'label' || style === 'simple') {
            return allowReserved ? v : encodeURIComponent(v);
        }
        return serializePrimitiveParam({
            allowReserved,
            name,
            value: v,
        });
    })
        .join(separator);
    return style === 'label' || style === 'matrix'
        ? separator + joinedValues
        : joinedValues;
};
const serializeObjectParam = ({ allowReserved, explode, name, style, value, }) => {
    if (style !== 'deepObject' && !explode) {
        let values = [];
        Object.entries(value).forEach(([key, v]) => {
            values = [
                ...values,
                key,
                allowReserved ? v : encodeURIComponent(v),
            ];
        });
        const joinedValues = values.join(',');
        switch (style) {
            case 'form':
                return `${name}=${joinedValues}`;
            case 'label':
                return `.${joinedValues}`;
            case 'matrix':
                return `;${name}=${joinedValues}`;
            default:
                return joinedValues;
        }
    }
    const separator = separatorObjectExplode(style);
    const joinedValues = Object.entries(value)
        .map(([key, v]) => serializePrimitiveParam({
        allowReserved,
        name: style === 'deepObject' ? `${name}[${key}]` : key,
        value: v,
    }))
        .join(separator);
    return style === 'label' || style === 'matrix'
        ? separator + joinedValues
        : joinedValues;
};
const defaultPathSerializer = ({ path, url: _url }) => {
    let url = _url;
    const matches = _url.match(PATH_PARAM_RE);
    if (matches) {
        for (const match of matches) {
            let explode = false;
            let name = match.substring(1, match.length - 1);
            let style = 'simple';
            if (name.endsWith('*')) {
                explode = true;
                name = name.substring(0, name.length - 1);
            }
            if (name.startsWith('.')) {
                name = name.substring(1);
                style = 'label';
            }
            else if (name.startsWith(';')) {
                name = name.substring(1);
                style = 'matrix';
            }
            const value = path[name];
            if (value === undefined || value === null) {
                continue;
            }
            if (Array.isArray(value)) {
                url = url.replace(match, serializeArrayParam({ explode, name, style, value }));
                continue;
            }
            if (typeof value === 'object') {
                url = url.replace(match, serializeObjectParam({
                    explode,
                    name,
                    style,
                    value: value,
                }));
                continue;
            }
            if (style === 'matrix') {
                url = url.replace(match, `;${serializePrimitiveParam({
                    name,
                    value: value,
                })}`);
                continue;
            }
            const replaceValue = encodeURIComponent(style === 'label' ? `.${value}` : value);
            url = url.replace(match, replaceValue);
        }
    }
    return url;
};
const createQuerySerializer = ({ allowReserved, array, object, } = {}) => {
    const querySerializer = (queryParams) => {
        let search = [];
        if (queryParams && typeof queryParams === 'object') {
            for (const name in queryParams) {
                const value = queryParams[name];
                if (value === undefined || value === null) {
                    continue;
                }
                if (Array.isArray(value)) {
                    search = [
                        ...search,
                        serializeArrayParam(Object.assign({ allowReserved, explode: true, name, style: 'form', value }, array)),
                    ];
                    continue;
                }
                if (typeof value === 'object') {
                    search = [
                        ...search,
                        serializeObjectParam(Object.assign({ allowReserved, explode: true, name, style: 'deepObject', value: value }, object)),
                    ];
                    continue;
                }
                search = [
                    ...search,
                    serializePrimitiveParam({
                        allowReserved,
                        name,
                        value: value,
                    }),
                ];
            }
        }
        return search.join('&');
    };
    return querySerializer;
};
exports.createQuerySerializer = createQuerySerializer;
const getAuthToken = async (auth, callback) => {
    const token = typeof callback === 'function' ? await callback(auth) : callback;
    if (!token) {
        return;
    }
    if (auth.scheme === 'bearer') {
        return `Bearer ${token}`;
    }
    if (auth.scheme === 'basic') {
        return `Basic ${btoa(token)}`;
    }
    return token;
};
exports.getAuthToken = getAuthToken;
const setAuthParams = async (_a) => {
    var _b;
    var { security } = _a, options = __rest(_a, ["security"]);
    for (const auth of security) {
        const token = await (0, exports.getAuthToken)(auth, options.auth);
        if (!token) {
            continue;
        }
        const name = (_b = auth.name) !== null && _b !== void 0 ? _b : 'Authorization';
        switch (auth.in) {
            case 'query':
                if (!options.query) {
                    options.query = {};
                }
                options.query[name] = token;
                break;
            case 'header':
            default:
                options.headers[name] = token;
                break;
        }
        return;
    }
};
exports.setAuthParams = setAuthParams;
const buildUrl = (options) => {
    const url = (0, exports.getUrl)({
        path: options.path,
        query: !options.paramsSerializer ? options.query : undefined,
        querySerializer: typeof options.querySerializer === 'function'
            ? options.querySerializer
            : (0, exports.createQuerySerializer)(options.querySerializer),
        url: options.url,
    });
    return url;
};
exports.buildUrl = buildUrl;
const getUrl = ({ path, query, querySerializer, url: _url, }) => {
    const pathUrl = _url.startsWith('/') ? _url : `/${_url}`;
    let url = pathUrl;
    if (path) {
        url = defaultPathSerializer({ path, url });
    }
    let search = query ? querySerializer(query) : '';
    if (search.startsWith('?')) {
        search = search.substring(1);
    }
    if (search) {
        url += `?${search}`;
    }
    return url;
};
exports.getUrl = getUrl;
const serializeFormDataPair = (formData, key, value) => {
    if (typeof value === 'string' || value instanceof Blob) {
        formData.append(key, value);
    }
    else {
        formData.append(key, JSON.stringify(value));
    }
};
const mergeConfigs = (a, b) => {
    const config = Object.assign(Object.assign({}, a), b);
    config.headers = (0, exports.mergeHeaders)(a.headers, b.headers);
    return config;
};
exports.mergeConfigs = mergeConfigs;
exports.axiosHeadersKeywords = [
    'common',
    'delete',
    'get',
    'head',
    'patch',
    'post',
    'put',
];
const mergeHeaders = (...headers) => {
    var _a;
    const mergedHeaders = {};
    for (const header of headers) {
        if (!header || typeof header !== 'object') {
            continue;
        }
        const iterator = Object.entries(header);
        for (const [key, value] of iterator) {
            if (exports.axiosHeadersKeywords.includes(key) &&
                typeof value === 'object') {
                mergedHeaders[key] = Object.assign(Object.assign({}, mergedHeaders[key]), value);
            }
            else if (value === null) {
                delete mergedHeaders[key];
            }
            else if (Array.isArray(value)) {
                for (const v of value) {
                    mergedHeaders[key] = [...((_a = mergedHeaders[key]) !== null && _a !== void 0 ? _a : []), v];
                }
            }
            else if (value !== undefined) {
                mergedHeaders[key] =
                    typeof value === 'object' ? JSON.stringify(value) : value;
            }
        }
    }
    return mergedHeaders;
};
exports.mergeHeaders = mergeHeaders;
exports.formDataBodySerializer = {
    bodySerializer: (body) => {
        const formData = new FormData();
        Object.entries(body).forEach(([key, value]) => {
            if (value === undefined || value === null) {
                return;
            }
            if (Array.isArray(value)) {
                value.forEach((v) => serializeFormDataPair(formData, key, v));
            }
            else {
                serializeFormDataPair(formData, key, value);
            }
        });
        return formData;
    },
};
exports.jsonBodySerializer = {
    bodySerializer: (body) => JSON.stringify(body),
};
const serializeUrlSearchParamsPair = (data, key, value) => {
    if (typeof value === 'string') {
        data.append(key, value);
    }
    else {
        data.append(key, JSON.stringify(value));
    }
};
exports.urlSearchParamsBodySerializer = {
    bodySerializer: (body) => {
        const data = new URLSearchParams();
        Object.entries(body).forEach(([key, value]) => {
            if (value === undefined || value === null) {
                return;
            }
            if (Array.isArray(value)) {
                value.forEach((v) => serializeUrlSearchParamsPair(data, key, v));
            }
            else {
                serializeUrlSearchParamsPair(data, key, value);
            }
        });
        return data;
    },
};
const createConfig = (override = {}) => (Object.assign({ baseURL: '' }, override));
exports.createConfig = createConfig;
//# sourceMappingURL=utils.js.map